package work.xuye.common.db.service;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import work.xuye.common.db.entity.Mapping;
import work.xuye.common.db.entity.Task;
import work.xuye.common.db.entity.Template;
import work.xuye.common.db.entity.vo.SinkParams;
import work.xuye.common.dto.TaskVO;

/**
 * @author xuye
 * @since 2023/3/4 18:44
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class TaskManager {

    private final TaskService taskService;
    private final TemplateService templateService;
    private final MappingService mappingService;


    @Cacheable(value = "snps_task", key = "#taskId", unless = "#result == null")
    public TaskVO getTaskInfoByTaskId(Integer taskId) {
        Task task = taskService.lambdaQuery()
                .eq(Task::getId, taskId)
                .one();
        Assert.notNull(task, "taskId" + taskId + "不存在");

        SinkParams sinkParams = task.getSinkParams();
        if (sinkParams != null) {
            String templateName = sinkParams.getTemplateName();

            Template template = templateService.lambdaQuery()
                    .eq(Template::getName, templateName)
                    .one();
            Assert.notNull(template, "模板" + templateName + "不存在");
            Mapping mapping = mappingService.lambdaQuery()
                    .eq(Mapping::getName, task.getName())
                    .one();
            Assert.notNull(mapping, "映射" + task.getName() + "不存在");

            return new TaskVO(task, template, mapping);
        } else {
            return new TaskVO(task, null, null);
        }

    }


}
