package work.xuye.sink.handler;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import work.xuye.common.constant.MessageConstants;
import work.xuye.common.db.entity.vo.SinkParams;
import work.xuye.common.db.entity.vo.TableTemplate;
import work.xuye.common.db.service.TaskManager;
import work.xuye.common.dto.TaskVO;
import work.xuye.common.enums.DeleteCheckMode;
import work.xuye.common.enums.MessageType;
import work.xuye.common.enums.Status;
import work.xuye.common.service.MessageService;
import work.xuye.sink.service.JdbcService;

import java.util.List;

/**
 * @author xuye
 * @since 2023/3/14 09:31
 **/
@Slf4j
@Component
@RequiredArgsConstructor
public class DisappearHandler {
    private final TaskManager taskManager;
    private final JdbcService jdbcService;
    private final MessageService messageService;

    public void handle(Message<List<String>> message) {

        Integer taskId = (Integer) message.getHeaders().get(MessageConstants.TASK_ID);
        TaskVO taskVO = taskManager.getTaskInfoByTaskId(taskId);
        String lastValue = taskVO.getTask().getName();
        SinkParams.CheckDelete delete = taskVO.getTask().getSinkParams().getCheckDelete();
        if (!delete.getMode().equals(DeleteCheckMode.disappear)) {
            return;
        }
        if (delete.getStatus().equals(Status.off)) {
            return;
        }
        String condition = delete.getDisappearConfig().getCondition();
        TableTemplate.Field uniqueField = taskVO.getTemplate().getTableTemplate().getUniqueField();
        String fieldName = uniqueField.getFieldName();
        String tableName = taskVO.getTask().getSinkParams().getTableName();
        String dataSourceName = taskVO.getTask().getSinkParams().getDataSourceName();

        String sql = SqlGenerator.generateGivenFieldSql(tableName, fieldName, condition, lastValue);
        List<String> dbs = jdbcService.getForList(dataSourceName, sql);
        dbs.removeAll(message.getPayload());
        List<String> msg = SqlGenerator.generateDeleteSqls(tableName, fieldName, dbs);
        StringBuilder builder = new StringBuilder();
        for (String deleteSql : msg) {
            builder.append(deleteSql).append("\n\r");
        }
        if (!ObjectUtils.isEmpty(msg)) {
            messageService.sendMessage(MessageType.info,
                    "检测到可能需要删除的数据", "删除条数: " + msg.size(),
                    builder.toString());
        }


    }
}
